@@ -408,9 +408,7 @@ sub AUTOLOAD  {
 
     *{$AUTOLOAD} = sub { return ($_[0]->_Value($Attrib))};
     return($self->_Value($Attrib));
-  }
-    
-  elsif ($AUTOLOAD =~ /.*::[sS]et_?(\w+)/o) {
+  } elsif ($AUTOLOAD =~ /.*::[sS]et_?(\w+)/o) {
 
     if ($self->_Accessible($1,'write')) {
       my $Attrib = $1;
@@ -432,8 +430,23 @@ sub AUTOLOAD  {
     else { 
       return(0, 'Nonexistant field?');
     }
+  } elsif ($AUTOLOAD =~ /.*::(\w+?)_?[oO]bj$/o) {
+    if ($self->_Accessible($1,'object')) {
+      my $Attrib = $1;
+      *{$AUTOLOAD} = sub {
+      	my $s = shift;
+      	return $s->_Object(
+      			Field => $Attrib,
+      			Args => [@_],
+      		);
+      };
+      return $self->_Object( Field => $Attrib, Args => [@_] );
+    } else {
+      return(0, 'No object mapping for field');
+    }
   }
 
+
   #Previously, I checked for writability here. but I'm not sure that's the
   #right idea. it breaks the ability to do ValidateQueue for a ticket
   #on creation.
@@ -565,14 +578,20 @@ overrid __Value.
 
 *__value = \&__Value;
 sub __Value {
- my $self = shift;
- my $field = shift;
+  my $self = shift;
+  my $field = lc(shift);
 
-  $field = lc $field;
-
-  return($self->{'values'}->{"$field"});
+  if (!$self->{'fetched'}{$field} and my $id = $self->{'values'}{'id'}) {
+    my $QueryString = "SELECT $field FROM " . $self->Table . " WHERE id = ?";
+    my $sth = $self->_Handle->SimpleQuery( $QueryString, $id );
+    my ($value) = eval { $sth->fetchrow_array() };
+    warn $@ if $@;
 
+    $self->{'values'}{$field} = $value;
+    $self->{'fetched'}{$field} = 1;
+  }
 
+  return($self->{'values'}{$field});
 }
 # }}}
 # {{{ sub _Value 
@@ -714,6 +733,50 @@ sub _Validate  {
   }	
 
 # }}}	
+# {{{ sub _Object 
+
+=head2 _Object
+
+_Object takes a single column name and an array reference.
+It creates new object instance of class specified in _ClassAccessable
+structure and calls LoadById on recently created object with the
+current column value as argument. It uses the array reference as
+the object constructor's arguments.
+Subclasses can override _Object to insert custom access control or
+define default contructor arguments.
+
+=cut
+
+sub _Object
+{
+  my $self = shift;
+  return $self->__Object(@_);
+}
+
+sub __Object
+{
+  my $self = shift;
+  my %args = ( Field => '', Args => [], @_ );
+
+  my $field = $args{'Field'};
+  my $class = $self->_Accessible( $field, 'object' );
+
+# Globs magic to be sure that we call 'eval "require $class"' only once
+# because eval is quite slow -- cubic@acronis.ru
+  no strict qw( refs );
+  my $vglob = ${ $class . '::' }{'VERSION'};
+  unless ( $vglob && *$vglob{'SCALAR'} ) {
+    eval "require $class";
+    die "Couldn't use $class: $@" if ( $@ );
+    unless ( $vglob && *$vglob{'SCALAR'} ) {
+      *{$class."::VERSION"} = '-1, By DBIx::SerchBuilder';
+    }
+  }
+
+  my $object = $class->new( @{ $args{'Args'} } );
+  $object->LoadById( $self->__Value( $field ) );
+  return $object;
+}
 
 # }}}
   
@@ -792,15 +855,17 @@ sub LoadByCols  {
 	if (defined $hash{$key} &&  $hash{$key} ne '') {
         my $op;
         my $value;
+	my $function = "?";
         if (ref $hash{$key} eq 'HASH') {
             $op = $hash{$key}->{operator};
             $value = $hash{$key}->{value};
+            $function = $hash{$key}->{function} || "?";
        } else {
             $op = '=';
             $value = $hash{$key};
         }
 
-		push @phrases, "$key $op ?"; 
+		push @phrases, "$key $op $function"; 
 		push @bind, $value;
 	}
 	else {
@@ -868,7 +933,11 @@ loaded values hash.
 sub LoadFromHash {
   my $self = shift;
   my $hashref = shift;
-  
+
+  foreach my $f ( keys %$hashref ) {
+      $self->{'fetched'}{lc $f} = 1;
+  }
+
   $self->{'values'} = $hashref;
   #$self->_DowncaseValuesHash();
   return ($self->{'values'}{'id'});
@@ -906,6 +975,11 @@ sub _LoadFromSQL {
         return ( 0, "Couldn't find row" );
     }
 
+    
+    foreach my $f ( keys %{$self->{'values'}||{}} ) {
+        $self->{'fetched'}{lc $f} = 1;
+    }
+
     #$self->_DowncaseValuesHash();
 
     ## I guess to be consistant with the old code, make sure the primary  
@@ -943,6 +1017,10 @@ sub _LoadFromSQLold {
         return ( 0, "Couldn't find row" );
     }
     
+    foreach my $f ( keys %{$fetched||{}} ) {
+        $self->{'fetched'}->{lc $f} = 1;
+    }
+    
     $self->{'values'} = $fetched;
 
     ## I guess to be consistant with the old code, make sure the primary  
